      SUBROUTINE BASDF
C
C     ******************************************************************
C     DEFINE KEY MODEL PARAMETERS
C     ******************************************************************
C
C        SPECIFICATIONS:
C     ------------------------------------------------------------------
      INCLUDE 'mfi2k.inc'
      CHARACTER*80 LINE
C     ------------------------------------------------------------------
C
C1------PRINT THE NAME OF THE PROGRAM.
      WRITE(IOUT,1)
    1 FORMAT(1H1,5X,'U.S. GEOLOGICAL SURVEY MODULAR',
     1      ' FINITE-DIFFERENCE GROUND-WATER FLOW MODEL')
C
C2------READ COMMENTS.
      CALL URDCOM(IUREAL(INBAS),LINE,41)
C
C4------LOOK FOR OPTIONS IN THE FIRST RECORD AFTER THE HEADING
20    IXSEC=0
      ICHFLG=0
      MODINP=0
      MODOUT=0
      LLOC=1
101   CALL URWORD(LINE,LLOC,ISTART,ISTOP,1,N,R,IOUT,INBAS)
      IF(LINE(ISTART:ISTOP).EQ.'XSECTION') THEN
         IXSEC=1
      ELSE IF(LINE(ISTART:ISTOP).EQ.'CHTOCH') THEN
         ICHFLG=1
      ELSE IF(LINE(ISTART:ISTOP).EQ.'FREE') THEN
         MODINP=1
         MODOUT=1
         WRITE(IOUT,4)
    4    FORMAT (1X,'THE FREE FORMAT OPTION HAS BEEN SELECTED')
      END IF
      IF(LLOC.LT.80) GO TO 101
      CALL IVLSAV('IXSEC',0,0,IXSEC)
      CALL IVLSAV('ICHFLG',0,0,ICHFLG)
C
C6------PRINT # OF LAYERS, ROWS, COLUMNS AND STRESS PERIODS.
      WRITE(IOUT,5) NLAY,NROW,NCOL
    5 FORMAT(1X,I4,' LAYERS',I10,' ROWS',I10,' COLUMNS')
      IF(NLAY.GT.200) THEN
         WRITE(IOUT,*) ' Too many layers -- maximum is 200'
         CALL RDABRT
      END IF
      IF(IXSEC.NE.0 .AND. NROW.NE.1) THEN
         WRITE(IOUT,*) ' Cannot specify a cross section unless NROW=1'
         CALL RDABRT
      END IF
      WRITE(IOUT,6) NPER
    6 FORMAT(1X,I3,' STRESS PERIOD(S) IN SIMULATION')
      IF(NPER.GT.MXPER) THEN
         WRITE(IOUT,*) ' Too many stress periods -- maximum is',MXPER
         CALL RDABRT
      END IF
      IF(NPER.LT.1) THEN
         WRITE(IOUT,*) ' There must be at least 1 stress period'
         CALL RDABRT
      END IF
      IF(IXSEC.NE.0) WRITE(IOUT,106)
106   FORMAT(1X,'CROSS SECTION OPTION IS SPECIFIED')
      IF(ICHFLG.NE.0) WRITE(IOUT,107)
107   FORMAT(1X,
     1    'CALCULATE FLOW BETWEEN ADJACENT CONSTANT-HEAD CELLS')
C
C
C8------RETURN
      RETURN
      END
      SUBROUTINE BASAL()
C     ******************************************************************
C     ALLOCATE SPACE FOR BASIC MODEL ARRAYS
C     ******************************************************************
C
C        SPECIFICATIONS:
C     ------------------------------------------------------------------
      INCLUDE 'mfi2k.inc'
C     ------------------------------------------------------------------
C
C1------PRINT A MESSAGE IDENTIFYING THE PACKAGE.
      WRITE(IOUT,1)INBAS
    1 FORMAT(1X,/1X,'BAS -- BASIC MODEL PACKAGE,',
     2' INPUT READ FROM UNIT',I3)
C
C8------RETURN
      RETURN
C
      END
      SUBROUTINE BASRP()
C     ******************************************************************
C     READ AND INITIALIZE BASIC MODEL ARRAYS
C     ******************************************************************
C
C        SPECIFICATIONS:
C     ------------------------------------------------------------------
      INCLUDE 'mfi2k.inc'
      CHARACTER*24 ANAME(2)
C
      DATA ANAME(1) /'          BOUNDARY ARRAY'/
      DATA ANAME(2) /'            INITIAL HEAD'/
C     ------------------------------------------------------------------
C  Get cross section code
      CALL IVLGET('IXSEC',0,0,ITYPE,IXSEC)
C
C2------READ BOUNDARY ARRAY(IBOUND) ONE LAYER AT A TIME.
      IF(IXSEC.EQ.0) THEN
         DO 100 K=1,NLAY
         KK=K
         CALL ARRMAK('IBOUND',KK,0,NCOL,NROW)
         CALL U2DINT(ANAME(1),NROW,NCOL,KK,INBAS,NARRAY,INBAS)
  100    CONTINUE
      ELSE
         CALL ARRMAK('IBOUND',0,0,NCOL,NLAY)
         CALL U2DINT(ANAME(1),NLAY,NCOL,-1,INBAS,NARRAY,INBAS)
      END IF
C
C3------READ AND PRINT HEAD VALUE TO BE PRINTED FOR NO-FLOW CELLS.
      IF(MODINP.EQ.0) READ(IUREAL(INBAS),'(F10.0)') HNOFLO
      IF(MODINP.NE.0) READ(IUREAL(INBAS),*) HNOFLO
      WRITE(IOUT,3) HNOFLO
    3 FORMAT(1X,/1X,'AQUIFER HEAD WILL BE SET TO ',1PG11.5,
     1       ' AT ALL NO-FLOW NODES (IBOUND=0).')
      CALL VALSAV('HNOFLO',0,0,HNOFLO)
C
C4------READ INITIAL HEADS.
      IF(IXSEC.EQ.0) THEN
         DO 300 K=1,NLAY
         KK=K
         CALL ARRMAK('SHEAD',KK,0,NCOL,NROW)
         CALL U2DREL(ANAME(2),NROW,NCOL,KK,INBAS,NARRAY,INBAS)
  300    CONTINUE
      ELSE
         CALL ARRMAK('SHEAD',0,0,NCOL,NLAY)
         CALL U2DREL(ANAME(2),NLAY,NCOL,-1,INBAS,NARRAY,INBAS)
      END IF
C
C9------RETURN
 1000 RETURN
      END
      SUBROUTINE SBASC()
C
C     ******************************************************************
C     CLOSE ALL FILES FROM WHICH AN EXISTING DATASET WAS READ.
C     ******************************************************************
C
C        SPECIFICATIONS:
C     ------------------------------------------------------------------
      INCLUDE 'mfi2k.inc'
C     ------------------------------------------------------------------
C
C  Only package files are read, so check each one and close those for
C  which there is a virtual unit.
      IF(INBAS.GT.0) CLOSE(UNIT=IUREAL(INBAS))
      DO 10 I=1,39
      IF(IUNIT(I).GT.0) CLOSE(UNIT=IUREAL(IUNIT(I)))
10    CONTINUE
C
      RETURN
      END
      SUBROUTINE BASSV(IUNDEF,IWRFLG)
C     ******************************************************************
C     SAVE DATA FOR BAS PACKAGE
C     ******************************************************************
C
C        SPECIFICATIONS:
C     ------------------------------------------------------------------
      INCLUDE 'mfi2k.inc'
      DIMENSION IUNITR(24)
      CHARACTER*80 LINE,MESS(1)
C     ------------------------------------------------------------------
      CTEXT(1)='Missing data for the BAS Package:'
      CTEXT(2)='------------------------------------------------------'
      NMISS=2
C  BAS Package is always active, so don't check for inactive
C
      DO 5 I=1,MXFNAM
      IF(IFNUSE(I).EQ.INBAS) IFNUSE(I)=0
5     CONTINUE
      IFNUSE(INBAS)=INBAS
      IF(IUREAL(INBAS).EQ.0) CALL REALUN(INBAS)
C
      IF(ILOUT.GT.0) THEN
         IF(IUREAL(ILOUT).EQ.0) CALL REALUN(ILOUT)
      END IF
      IF(IGOUT.GT.0) THEN
         IF(IUREAL(IGOUT).EQ.0) CALL REALUN(IGOUT)
      END IF
C
      IUNDEF=0
      IFIL=IUMISC
      IF(IWRFLG.LE.0) IFIL=0
      IF(IWRFLG.EQ.1) OPEN(FILE='MFI.BAS',UNIT=IFIL)
      IF(IWRFLG.GT.1) OPEN(FILE=FILNAM(INBAS),UNIT=IFIL)
      CALL IVLGET('IXSEC',0,0,ITYPE,IXSEC)
      IF(ITYPE.NE.0) IXSEC=0
      CALL IVLGET('ICHFLG',0,0,ITYPE,ICHFLG)
      IF(ITYPE.NE.0) ICHFLG=0
      IF(IWRFLG.GT.0) THEN
         DO 8 I=1,20
         IF(COMENT(I,41)(1:1).EQ.'#') THEN
            DO 6 N=80,1,-1
            IF(COMENT(I,41)(N:N) .NE. ' ') GO TO 7
6           CONTINUE
7           WRITE(IFIL,'(A)') COMENT(I,41)(1:N)
         END IF
8        CONTINUE
      END IF
C
C  Write options record
      IF(IWRFLG.GT.0) THEN
         LL=0
         IF(MODOUT.NE.0) THEN
            LINE(LL+1:LL+4)='FREE'
            LL=LL+4
         END IF
         IF(ICHFLG.NE.0) THEN
            LINE(LL+1:LL+7)= ' CHTOCH'
            LL=LL+7
         END IF
         IF(IXSEC.NE.0) THEN
            LINE(LL+1:LL+9)= ' XSECTION'
            LL=LL+9
         END IF
         IF(LL.EQ.0) WRITE(IFIL,'(A)') 'NO OPTIONS'
         IF(LL.GT.0) WRITE(IFIL,'(A)') LINE(1:LL)
      END IF
C
      IF(IXSEC.EQ.0) THEN
         DO 10 K=1,NLAY
         KK=K
         CALL U2DISV('IBOUND',KK,0,IUNDEF,IFIL,INBAS)
         IF(IUNDEF.NE.0 .AND. NMISS.LT.45 .AND. IWRFLG.LT.0) THEN
            NMISS=NMISS+1
            IUNDEF=0
            WRITE(CTEXT(NMISS),'(A,I10)') 'IBOUND    ',KK
         END IF
10       CONTINUE
      ELSE
         CALL U2DISV('IBOUND',0,0,IUNDEF,IFIL,INBAS)
         IF(IUNDEF.NE.0 .AND. NMISS.LT.45 .AND. IWRFLG.LT.0) THEN
            NMISS=NMISS+1
            IUNDEF=0
            WRITE(CTEXT(NMISS),'(A)') 'IBOUND              '
         END IF
      END IF
C
      CALL VALGET('HNOFLO',0,0,ITYPE,HNOFLO)
      IF(ITYPE.NE.0) HNOFLO=999.99
      IF(IWRFLG.GT.0) THEN
         IF(ABS(HNOFLO).LE.999999. .AND. ABS(HNOFLO).GE.100.) THEN
            WRITE(IFIL,'(F10.2,A)') HNOFLO,'  HNOFLO'
         ELSE
            WRITE(IFIL,'(1P,E10.3,A)') HNOFLO,'  HNOFLO'
         END IF
      END IF
C
      IF(IXSEC.EQ.0) THEN
         DO 20 K=1,NLAY
         KK=K
         CALL U2DRSV('SHEAD',KK,0,IUNDEF,IFIL,INBAS,'SHEAD',KK)
         IF(IUNDEF.NE.0 .AND. NMISS.LT.45 .AND. IWRFLG.LT.0) THEN
            NMISS=NMISS+1
            IUNDEF=0
            WRITE(CTEXT(NMISS),'(A,I8)') 'INITIAL HEAD',KK
         END IF
20       CONTINUE
      ELSE
         CALL U2DRSV('SHEAD',0,0,IUNDEF,IFIL,INBAS,'SHEAD',0)
         IF(IUNDEF.NE.0 .AND. NMISS.LT.45 .AND. IWRFLG.LT.0) THEN
            NMISS=NMISS+1
            IUNDEF=0
            WRITE(CTEXT(NMISS),'(A)') 'INITIAL HEAD        '
         END IF
      END IF
C
      IF(IWRFLG.GT.0) CLOSE(UNIT=IFIL)
C
      IF(IWRFLG.LT.0) THEN
         IF(NMISS.GT.2) THEN
            CALL TXTLOG(CTEXT,NMISS)
            IUNDEF=1
         ELSE
            MESS(1)='All data are defined for the BAS Package'
            CALL TXTLOG(MESS,1)
         END IF
      END IF
C
      CALL FNMRMV
      RETURN
      END
